/*
shutdown.p

AUTHOR : Cyril Kern
MAIL : cyril.kern@free.fr

GAME : SHUTDOWN

AIM : Do not let the cube shutdown. 

Rules :
- When a side is green all is ok. 
- When a side goes blue, the cube starts to shutdown.
- If you tap the side while blue, you awake it and it returns to green.
- If the face goes red, the face shuts down and cannot be reactivated.
- You lose the game when more than 3 faces are shutdowned

Information :
- Tapping green side loses points
- Keeping the more side activated gets greater points

*/

#include <futurocube>

//face status
#define GOOD_STATUS 0
#define CHANGING_STATUS 10
#define BAD_STATUS 20

new icon[]=[ICON_MAGIC1,ICON_MAGIC2,2,5, 0x00FF0000,0xFF000000,0x00FF0000,0xFF000000,0x00FF0000,0xFF000000,0x00FF0000, 0xFF000000,0x00FF0000,  '''','''']
new timerMax	// the disabling time of a face
new timerDeactivate	// time between the desactivation of 2 sides
new faceStatus[6]	//the status of the face
new faceTimeCreation[6]	//the desactivation time of a face

new nbDisable	//the number of face that have been disabled
new score		//the score

main()
{
	ICON(icon)
	RegAllSideTaps() 	//register tap
	initCube()			//init game
	SetIntensity(128)	//set intensity
	new motion			//motion check

	new lost = false
	new start = false
	
	new colorAnim = 0
	
	Delay(300)
	
	while(colorAnim<255) {
		Sleep()
		SetRgbColor(0, colorAnim, 0)
		DrawCube()
		PrintCanvas()
		colorAnim+=3
	}
	
	
	draw()
		
	Play("_g_TAPTOSTART")
	//wait for tapping to start the game
	while (!start) {
		Sleep()
		motion = Motion()
		if (motion)
		{
			if(eTapSide()) {
				start = true
				Play("ballhit")
			}
			AckMotion()
		}
	}

	//start the deactivation timer
	SetTimer(6,timerDeactivate) 
	
	//while not lost
	while( !lost ) {

		Sleep()

		motion=Motion()

		//get taped side
		if (motion) 
		{
			tapSide(eTapSide())
			AckMotion()			
		}
		
		//if we timer 6 reaches 0, we disable a new side
		if (GetTimer(6) == 0) 
		{
			disableSide()
			SetTimer(6, timerDeactivate)
		}
		
		//update side status
		updateStatus()
		
		//draw the cube
		draw()
		
		//test lost
		lost = (countFaceStatus(BAD_STATUS) == 4)
	}
	
	
	new animationFinished = false;
	new loopFlash = 0
	SetTimer(7, 200)
	
	while (!animationFinished)
	{
		Sleep()

		if (GetTimer(7) <= 0) {
			if (loopFlash % 2 == 0) {
				SetRgbColor(255,0,0)
				DrawCube()
				PrintCanvas()
			}
			else {
				SetRgbColor(0,0,0)
				DrawCube()
				PrintCanvas()
			}
			loopFlash++
			SetTimer(7, 200)
		}
		
		animationFinished = loopFlash > 7
	}
	
	Delay(600)
	
	
	
	//display score
	Score(score, SCORE_NORMAL, 1, 1)


}

//return the number of faces of a specific status
countFaceStatus(status) {
	new i
	new res = 0
	for (i = 0; i < 6; i++) {
		if (faceStatus[i] == status)
			res++
	}
	return res
}

//update the status of the face
updateStatus() {
	new i
	for (i = 0; i < 6; i++) {
		if (faceStatus[i] == CHANGING_STATUS && GetTimer(i) <= 0) {
			faceStatus[i] = BAD_STATUS
			Play("uff")
		}
	}
}

//select randomly a side and start disabling it
disableSide() {

	new bad = countFaceStatus(BAD_STATUS)
	new change = countFaceStatus(CHANGING_STATUS)
	
	if (bad + change < 6) {		//if we still have good face
	
		//get a random face
		new random = GetRnd(6)
		while (faceStatus[random] != GOOD_STATUS)
			random = GetRnd(6)
		
		//init the timer
		SetTimer(random, timerMax)

		//set the new status
		faceStatus[random] = CHANGING_STATUS
		faceTimeCreation[random] = timerMax
		nbDisable++
		Play("beep")
		
		//disable time between creation depending on number of active sides
		if (nbDisable % 5 == 0) {
			if (timerMax > (4 - countFaceStatus(BAD_STATUS)) * 20)
				timerMax -= 100
		}
	}
}

//detect the tap on a side
tapSide(side) {

	//not allowed to enable an already activated face
	if (faceStatus[side] == GOOD_STATUS) {
		updateScore(-10)
		Play("bubble")
	}
	
	//if face is desactivating
	if (faceStatus[side] == CHANGING_STATUS) {
		updateScore(6 - countFaceStatus(BAD_STATUS))	//score depends on number of bad faces
		Play("ballhit")
	}
		
	// reactivate face if not in BAD_STATUS
	if (faceStatus[side] != BAD_STATUS) {
		faceStatus[side] = GOOD_STATUS
		if (timerDeactivate > 50)
			timerDeactivate -= 50
	}
}

//compute score
updateScore(val) {
	if (score + val > 0)
		score = score + val
}


//init the cube with default values
initCube() {
	score = 0
	nbDisable = 0
	timerMax = 3000
	timerDeactivate = 2000
 
	new i
	
	for (i=0; i<6; i++) {
		faceStatus[i] = GOOD_STATUS
	}
}

//draw the cube
draw() {
	new i
	new intens
	new time
	for (i=0; i<6; i++) {

		
		if (faceStatus[i] == GOOD_STATUS) {	//GOOD_STATUS then green side
			SetRgbColor(0,255,0)
		}
		
		if (faceStatus[i] == CHANGING_STATUS) {	//currenlty disabling, set a gradient from blue to red
			time = GetTimer(i)
			intens = 255 - (time * 255 / faceTimeCreation[i])		
			SetRgbColor(intens,0,255-intens)

		}
		
		if (faceStatus[i] == BAD_STATUS) {		//face down, black
			SetRgbColor(0,0,0)
		}

		DrawSide(i)
	}
	PrintCanvas()
}

